package org.hibernate.internal;

import org.hibernate.*;
import org.hibernate.cfg.*;
import org.hibernate.engine.spi.*;
import org.hibernate.id.*;
import org.hibernate.internal.util.*;
import org.hibernate.mapping.*;
import org.hibernate.mapping.Collection;
import org.hibernate.persister.collection.CollectionPersister;
import org.hibernate.persister.entity.*;
import org.hibernate.persister.spi.*;
import org.hibernate.service.*;
import java.util.*;
import java.util.Map;

public class MutableSessionFactoryImpl
extends SessionFactoryImpl
{
   private final static transient Map<String, String> dynamicImports =
                                                   new HashMap<String, String>();

   public MutableSessionFactoryImpl(final Configuration cfg,
                                    Mapping mapping,
                                    ServiceRegistry serviceRegistry,
                                    Settings settings,
                                    SessionFactoryObserver observer)
   throws HibernateException
   {
      super(cfg, mapping, serviceRegistry, settings, observer);
   }

   public void add(PersistentClass persistentClass, Configuration configuration)
   {
      String entityName = persistentClass.getEntityName();

      IdentifierGenerator generator = persistentClass.getIdentifier().createIdentifierGenerator(
						configuration.getIdentifierGeneratorFactory(),
						getDialect(),
				      settings.getDefaultCatalogName(),
				      settings.getDefaultSchemaName(),
				      (RootClass) persistentClass
	   );
      identifierGenerators.put(entityName, generator);

      PersisterFactory persisterFactory = serviceRegistry.getService(PersisterFactory.class);

      EntityPersister entityPersister = persisterFactory.createEntityPersister(
                                                          persistentClass,
                                                          null, //TODO accessStrategy?
                                                          null, //TODO naturalIdAccessStrategy?
                                                          this,
                                                          configuration.getMapping());

      entityPersisters.put(entityName, entityPersister);
      entityPersister.postInstantiate();
      registerEntityNameResolvers(entityPersister);

      // map secondary tables
      Iterator<Property> iter = persistentClass.getPropertyIterator();
      while (iter.hasNext())
      {
         Property property = iter.next();
         if (property.getType().isCollectionType())
         {
            Collection collection = (Collection) property.getValue();
            CollectionPersister persister = persisterFactory.createCollectionPersister(
					                                            configuration,
					                                            collection,
					                                            null, //TODO accessStrategy?
					                                            this);
            collectionPersisters.put(collection.getRole(), persister);
         }
      }

      synchronized (dynamicImports)
      {
         dynamicImports.put(entityName, entityName);
         dynamicImports.put(StringHelper.unqualify(entityName), entityName);
      }
   }

   public void remove(PersistentClass persistentClass)
   {
      String entityName = persistentClass.getEntityName();

      identifierGenerators.remove(entityName);
      EntityPersister persister = entityPersisters.remove(entityName);

      EntityNameResolver[] resolvers =
               persister.getEntityMetamodel().getTuplizer().getEntityNameResolvers();
      if (resolvers != null)
      {
         for (EntityNameResolver resolver : resolvers)
            entityNameResolvers.remove(resolver);
      }

      // clear secondary tables
      Iterator<Property> iter = persistentClass.getPropertyIterator();
      while (iter.hasNext())
      {
         Property property = iter.next();
         if (property.getType().isCollectionType())
         {
            Collection collection = (Collection) property.getValue();
            collectionPersisters.remove(collection.getRole());
         }
      }

      synchronized (dynamicImports)
      {
         dynamicImports.remove(entityName);
         dynamicImports.remove(StringHelper.unqualify(entityName));
      }
   }

   public String getImportedClassName(String className)
   {
      String res = super.getImportedClassName(className);
      if (res == null)
      {
         synchronized (dynamicImports)
         {
            res = dynamicImports.get(className);
         }
      }

      return res;
   }
}
